# ----------------------------------------------------------------
#   ajaxtb-main.pl ---- Ajax track back component (CGI)
#   Copyright 2004-2006 Kawasaki Yusuke <u-suke [at] kawa.net>
#   http://www.kawa.net/works/ajax/ajaxtb/ajaxtb.html
#   http://www.sixapart.jp/movabletype/manual/mttrackback.html
# ----------------------------------------------------------------
    use strict;
    use CGI;
    use XML::FeedPP;
    require "misclib.pl";
# ----------------------------------------------------------------
    my $TB_PARAMS   = [qw( charset url title excerpt blog_name )];
    my $AJAXTB_DATA = "ajaxtb-data";
    my $URL_ROOT     = "http://www.kawa.net";
    my $DOC_ROOT     = "/home10/u-suke/www.kawa.net";
# ----------------------------------------------------------------
    &ajaxtb_main();
# ----------------------------------------------------------------
sub ajaxtb_main {
    my $cgi = new CGI();
    my $query = $cgi->Vars();

    if ( $query->{"__mode"} eq "rss" ) {
        &ajaxtb_mode_rss();
        return;
    };

    my $date = &misclib::get_w3cdtf_date();
    my $iphash = &misclib::get_ipaddr_hash();
    my $path = $ENV{PATH_INFO} if exists $ENV{PATH_INFO};
    my $method = $ENV{REQUEST_METHOD} if exists $ENV{REQUEST_METHOD};

    if ( $path !~ m#^/# ) {
        &misclib::tb_err_response( "invalid trackback url" );
    } elsif ( $method ne "POST" ) {
        &misclib::redirect( $URL_ROOT.$path );
    } elsif ( $query->{url} !~ m#^https?://\w# ) {
        &misclib::tb_err_response( "page url is not given" );
    } elsif ( $query->{url} !~ m#^https?://([\w\-]+\.)+\w+(/[\041-\176]*)?$# ) {
        &misclib::tb_err_response( "page url is not valid" );
    } elsif ( $query->{title} eq "" ) {
        &misclib::tb_err_response( "page title is not given" );
    }

    my $file = &misclib::get_log_filename( $AJAXTB_DATA );
    &misclib::tb_err_response( "invalid service url" ) unless defined $file;

    my $line = [ $date, $$, $iphash ];
    my $charset = $query->{charset};
    foreach my $key ( @$TB_PARAMS ) {
        my $val = $query->{$key};
        if ( defined $val && defined $charset ) {
            &misclib::encode_from_to( \$val, $charset, 'utf8' );
        }
        push( @$line, $val );
    }
    my $werror = &misclib::write_tab_line( $file, $line );
    &misclib::tb_err_response( $werror ) if defined $werror;

    &ajaxtb_update_rss();

    &misclib::tb_ok_response();
}
# ----------------------------------------------------------------
sub ajaxtb_mode_rss {
    my $rssfile = &ajaxtb_update_rss();
    &misclib::tb_err_response( "rss update failed." ) unless $rssfile;
    open( RSS, $rssfile ) or &misclib::tb_err_response( "$! - $rssfile" );
    print "Content-Type: text/xml; charset=UTF-8\n\n";
    print <RSS>;
    close( RSS );
}
# ----------------------------------------------------------------
sub ajaxtb_update_rss {
    my $max_total = shift || 50;
    my $max_page  = shift || 10;
    my $max_json  = shift || 10;

    my $bufline = &misclib::find_recent_lines( $AJAXTB_DATA, $max_total, $max_page );
    my $feed = XML::FeedPP::RSS->new();

    $feed->link( $URL_ROOT );
    $feed->title( "Trackbacks $URL_ROOT" );
    my $alias = &misclib::read_url_alias( $AJAXTB_DATA ) || {};
    $alias = { reverse %$alias };

    foreach my $pair ( @$bufline ) {
        my $url = ( $pair->[0] =~ m#([^/]+)\.txt$# )[0];
        my $path;
        if ( $alias->{$url} ) {
            $path = $alias->{$url};
        } else {
            $path = $url;
            $path =~ s#_#/#g;           # if aliases is not exist
        }
        my $file = "$DOC_ROOT/$path";
        next unless ( -r $file );
        my( $pubdate, $pid, $ip, $charset, $url, $title,
            $except, $blog_name ) = split( /\t/, $pair->[1] );
        chomp $blog_name;
        my $url = "$URL_ROOT/$path#tb-$pubdate";
        my $item = $feed->add_item( $url );
        $item->title( $title );
        $item->description( $except );
        $item->pubDate( $pubdate );
        $item->author( $blog_name );
    }

    my $rssfile = "$AJAXTB_DATA/recent.xml";
    &misclib::output_feed_rss( $feed, $rssfile );

    my $jsonfile = "$AJAXTB_DATA/recent.json";
    $feed->limit_item($max_json);
    &misclib::output_feed_json( $feed, $jsonfile );

    $rssfile;
}
# ------------------------------------------------------------------------
;1;
# ------------------------------------------------------------------------
